clc

% Revised on December 14, 2012

% This is the program for the estimation of alpha-MEU model
% There are two model parameters: 
%      ambiguity attitude:  0 < alpha <= 1
%      (constant absolute) risk aversion:   0 <= rho <= 5

load AllData.mat;

% AllData is (50xN)x8 matrix of the experimental data 
% from Ahn, D., S. Choi, D. Gale, and S. Kariv (2013),
% where N is the number of subjects.
% AllData = [ID Round D1 D2 D3 Intercept1 Intercept2 Intercept3]
%       where Di is a token allocation at ith state.
% state 2 is unambiguous.

dt = AllData; 

n = length(dt)/50; 

% nr is the number of replication in the bootstrap method of computing
%    the standard errors of model parameters.

nr = 500;

Estimates = []; 
PreD_Alpha = []; 
bootsample = [];

for i = 1:n
    
    m1 = 50*(i-1)+1; m2 = 50*i; 
    id = dt(m1,1)

    % y and ymax are defined as demand and intercept at unambiguous state
   
    x = dt(m1:m2,3); y = dt(m1:m2,4); z = dt(m1:m2,5);
    xmax = dt(m1:m2,6); ymax = dt(m1:m2,7); zmax = dt(m1:m2,8);

    % Parameter Restrictions
    % 0.01 <= alpha <= 0.99, 0 <= rho <= 5
    
    A = [-1 0; 1 0; 0 -1; 0 1];
    b = [-0.01;0.99;0;5];
    
    options = optimset('Algorithm','interior-point','Display','iter','MaxFunEvals',1.0E+5);
    
    % Stage 1: estimating individual model parameters
    
    pm01 = [1/2;0.05];
    
    [pm1,tssr1] = fminsearch(@ssr,pm01,options,y,x,z,ymax,xmax,zmax);
    
    pm02 = [1/2;0.2];
    
    [pm2,tssr2] = fminsearch(@ssr,pm02,options,y,x,z,ymax,xmax,zmax);

    pm03 = [0.35;0.1];
    
    [pm3,tssr3] = fminsearch(@ssr,pm03,options,y,x,z,ymax,xmax,zmax);
    
    pm04 = [0.65;0.1];
    
    [pm4,tssr4] = fminsearch(@ssr,pm04,options,y,x,z,ymax,xmax,zmax);

    pm05 = [0.6;0.02];
    
    [pm5,tssr5] = fminsearch(@ssr,pm05,options,y,x,z,ymax,xmax,zmax);
    
    pm06 = [0.35;0.1];
    
    [pm6,tssr6] = fminsearch(@ssr,pm06,options,y,x,z,ymax,xmax,zmax);
    
    pm07 = [0.4;0.02];
    
    [pm7,tssr7] = fminsearch(@ssr,pm07,options,y,x,z,ymax,xmax,zmax);

    pm08 = [0.8;0.02];
    
    [pm8,tssr8] = fminsearch(@ssr,pm08,options,y,x,z,ymax,xmax,zmax);
       
    tssrmin = min([tssr1 tssr2 tssr3 tssr4 tssr5 tssr6 tssr7 tssr8]);
    
    if (tssr1 <= tssrmin)
        
        pm = pm1; tssr = tssrmin;
    
    elseif (tssr2 <= tssrmin)
        
        pm = pm2; tssr = tssrmin;
        
    elseif (tssr3 <= tssrmin)
        
        pm = pm3; tssr = tssrmin;
        
    elseif (tssr4 <= tssrmin)
        
        pm = pm4; tssr = tssrmin;
        
    elseif (tssr5 <= tssrmin)
        
        pm = pm5; tssr = tssrmin;
        
    elseif (tssr6 <= tssrmin)
        
        pm = pm6; tssr = tssrmin;
        
    elseif (tssr7 <= tssrmin)
        
        pm = pm7; tssr = tssrmin;
       
    else 
        
        pm = pm8; tssr = tssrmin;

    end

    alpha = pm(1); rho = pm(2);
    
    % Stage 2: Computing predicted demands using the estimated model
    %          parameters
    
    for j = 1:50
    
        p1 = 1/xmax(j); p2 = 1/ymax(j); p3 = 1/zmax(j);
        
        pd = demand(p1,p2,p3,alpha,rho);
        
        PreD_Alpha = [PreD_Alpha;i,id,j,pd(1),pd(2),pd(3),xmax(j),ymax(j),zmax(j)];
    
    end
    
    % Stage 2: Estimating the covariance matrix using the bootstrap method
    
    btsample = bstrap(pm,x,y,z,xmax,ymax,zmax,nr);
    
    bootsample = [bootsample; id*ones(nr,1),btsample];
    
    %%% To construct the Wald statistic for the null hypothesis that beta1 = 1/3 and beta2 = 1/3
    
    cov_pms = cov(btsample);
    
    Estimates = [Estimates;id,alpha,cov_pms(1),rho,cov_pms(2),tssr];
    
    save ESTM_AlphaMEU_Jan312013_v2 Estimates
    save PreD_AlphaMEU_Jan312013_v2 PreD_Alpha
    save Bootsample_AlphaMEU_Jan312013_v2 bootsample
     
end 





